@@ -1,6 +1,7 @@
 import logging
 import os
 
+import concurrent.futures
 import requests
 from django.core.management.base import CommandError
 from le_utils.constants import content_kinds
@@ -11,9 +12,8 @@ from ...utils import transfer
 from kolibri.core.content.errors import InvalidStorageFilenameError
 from kolibri.core.content.models import ContentNode
 from kolibri.core.content.utils.file_availability import LocationError
-from kolibri.core.content.utils.import_export_content import calculate_files_to_transfer
 from kolibri.core.content.utils.import_export_content import compare_checksums
-from kolibri.core.content.utils.import_export_content import get_nodes_to_transfer
+from kolibri.core.content.utils.import_export_content import get_import_export_data
 from kolibri.core.content.utils.paths import get_channel_lookup_url
 from kolibri.core.content.utils.upgrade import get_import_data_for_update
 from kolibri.core.tasks.management.commands.base import AsyncCommand
@@ -203,7 +203,11 @@ class Command(AsyncCommand):
     ):
         try:
             if not import_updates:
-                nodes_for_transfer = get_nodes_to_transfer(
+                (
+                    total_resource_count,
+                    files_to_download,
+                    total_bytes_to_transfer,
+                ) = get_import_export_data(
                     channel_id,
                     node_ids,
                     exclude_node_ids,
@@ -212,17 +216,6 @@ class Command(AsyncCommand):
                     drive_id=drive_id,
                     peer_id=peer_id,
                 )
-                total_resource_count = (
-                    nodes_for_transfer.exclude(kind=content_kinds.TOPIC)
-                    .values("content_id")
-                    .distinct()
-                    .count()
-                )
-
-                (
-                    files_to_download,
-                    total_bytes_to_transfer,
-                ) = calculate_files_to_transfer(nodes_for_transfer, False)
             else:
                 (
                     total_resource_count,
@@ -285,11 +278,10 @@ class Command(AsyncCommand):
         with self.start_progress(
             total=total_bytes_to_transfer + dummy_bytes_for_annotation
         ) as overall_progress_update:
-            exception = None  # Exception that is not caught by the retry logic
-
             if method == DOWNLOAD_METHOD:
                 session = requests.Session()
 
+            file_transfers = []
             for f in files_to_download:
 
                 if self.is_cancelled():
@@ -318,6 +310,7 @@ class Command(AsyncCommand):
                     filetransfer = transfer.FileDownload(
                         url, dest, session=session, cancel_check=self.is_cancelled
                     )
+                    file_transfers.append((f, filetransfer))
                 elif method == COPY_METHOD:
                     try:
                         srcpath = paths.get_content_storage_file_path(
@@ -330,37 +323,57 @@ class Command(AsyncCommand):
                     filetransfer = transfer.FileCopy(
                         srcpath, dest, cancel_check=self.is_cancelled
                     )
+                    file_transfers.append((f, filetransfer))
 
-                try:
-                    status = self._start_file_transfer(
-                        f, filetransfer, overall_progress_update
-                    )
+            with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
+                batch_size = 100
+                # ThreadPoolExecutor allows us to download files concurrently,
+                # greatly reducing download time in most cases. However, loading
+                # all the downloads into the pool requires considerable memory,
+                # so we divide the downloads into batches to keep memory usage down.
+                # In batches of 100, total RAM usage doesn't exceed 250MB in testing.
+                while len(file_transfers) > 0:
+                    future_file_transfers = {}
+                    for i in range(batch_size):
+                        if len(file_transfers) > 0:
+                            f, filetransfer = file_transfers.pop()
+                            future = executor.submit(
+                                self._start_file_transfer, f, filetransfer
+                            )
+                            future_file_transfers[future] = (f, filetransfer)
 
-                    if self.is_cancelled():
-                        break
+                    for future in concurrent.futures.as_completed(
+                        future_file_transfers
+                    ):
+                        f, filetransfer = future_file_transfers[future]
+                        try:
+                            status, data_transferred = future.result()
+                            overall_progress_update(data_transferred)
+                            if self.is_cancelled():
+                                break
 
-                    if status == FILE_SKIPPED:
-                        number_of_skipped_files += 1
-                    else:
-                        file_checksums_to_annotate.append(f.id)
-                        transferred_file_size += f.file_size
-                except transfer.TransferCanceled:
-                    break
-                except Exception as e:
-                    logger.error(
-                        "An error occurred during content import: {}".format(e)
-                    )
-                    if (
-                        isinstance(e, requests.exceptions.HTTPError)
-                        and e.response.status_code == 404
-                    ) or (isinstance(e, OSError) and e.errno == 2):
-                        # Continue file import when the current file is not found from the source and is skipped.
-                        overall_progress_update(f.file_size)
-                        number_of_skipped_files += 1
-                        continue
-                    else:
-                        exception = e
-                        break
+                            if status == FILE_SKIPPED:
+                                number_of_skipped_files += 1
+                            else:
+                                file_checksums_to_annotate.append(f.id)
+                                transferred_file_size += f.file_size
+                        except transfer.TransferCanceled:
+                            break
+                        except Exception as e:
+                            logger.error(
+                                "An error occurred during content import: {}".format(e)
+                            )
+                            if (
+                                isinstance(e, requests.exceptions.HTTPError)
+                                and e.response.status_code == 404
+                            ) or (isinstance(e, OSError) and e.errno == 2):
+                                # Continue file import when the current file is not found from the source and is skipped.
+                                overall_progress_update(f.file_size)
+                                number_of_skipped_files += 1
+                                continue
+                            else:
+                                self.exception = e
+                                break
 
             with db_task_write_lock:
                 annotation.set_content_visibility(
@@ -395,33 +408,31 @@ class Command(AsyncCommand):
 
             overall_progress_update(dummy_bytes_for_annotation)
 
-            if exception:
-                raise exception
+            if self.exception:
+                raise self.exception
 
             if self.is_cancelled():
                 self.cancel()
 
-    def _start_file_transfer(self, f, filetransfer, overall_progress_update):
+    def _start_file_transfer(self, f, filetransfer):
         """
         Start to transfer the file from network/disk to the destination.
         Return value:
             * FILE_TRANSFERRED - successfully transfer the file.
             * FILE_SKIPPED - the file does not exist so it is skipped.
         """
-        with filetransfer, self.start_progress(
-            total=filetransfer.total_size
-        ) as file_dl_progress_update:
+        data_transferred = 0
+
+        with filetransfer:
             for chunk in filetransfer:
-                length = len(chunk)
-                overall_progress_update(length)
-                file_dl_progress_update(length)
+                data_transferred += len(chunk)
 
             # Ensure that if for some reason the total file size for the transfer
             # is less than what we have marked in the database that we make up
             # the difference so that the overall progress is never incorrect.
             # This could happen, for example for a local transfer if a file
             # has been replaced or corrupted (which we catch below)
-            overall_progress_update(f.file_size - filetransfer.total_size)
+            data_transferred += f.file_size - filetransfer.total_size
 
             # If checksum of the destination file is different from the localfile
             # id indicated in the database, it means that the destination file
@@ -432,9 +443,9 @@ class Command(AsyncCommand):
                 e = "File {} is corrupted.".format(filetransfer.source)
                 logger.error("An error occurred during content import: {}".format(e))
                 os.remove(filetransfer.dest)
-                return FILE_SKIPPED
+                return FILE_SKIPPED, data_transferred
 
-        return FILE_TRANSFERRED
+        return FILE_TRANSFERRED, data_transferred
 
     def handle_async(self, *args, **options):
         if options["command"] == "network":
